"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var chalk = require("chalk");
var fs = require("fs");
var inquirer = require("inquirer");
var path = require("path");
var configuration_1 = require("./configuration");
var error_1 = require("./error");
var lint_1 = require("./lint");
var svn_1 = require("./svn");
var svn = new svn_1.Svn();
var lint = new lint_1.Lint();
function run(options, logger) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var error_2, outputs;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    _a.trys.push([0, 2, , 3]);
                    return [4 /*yield*/, runWorker(options, logger)];
                case 1: return [2 /*return*/, _a.sent()];
                case 2:
                    error_2 = _a.sent();
                    if (error_2 instanceof error_1.SystemError) {
                        logger.error(error_2.message + "\n");
                        return [2 /*return*/, 1 /* SystemError */];
                    }
                    if (error_2 instanceof error_1.LintError) {
                        outputs = error_2.message.split("\n\n");
                        outputs.forEach(function (output) {
                            logger.error(chalk.red(output + "\n\n"));
                        });
                        return [2 /*return*/, 2 /* LintError */];
                    }
                    throw error_2;
                case 3: return [2 /*return*/];
            }
        });
    });
}
exports.run = run;
function runWorker(options, logger) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var _a, stdout, stderr, changesStr, changes, chooses, lintCheckResult, input, needAddItems, error_3;
        return tslib_1.__generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    _b.trys.push([0, 10, , 11]);
                    if (!checkFileExist(configuration_1.PROJECT_NAME, options.project)) {
                        throw new error_1.SystemError("\u627E\u4E0D\u5230\u811A\u672C\u68C0\u6D4B\u6587\u4EF6: " + (options.project === undefined ? configuration_1.PROJECT_NAME : options.project));
                    }
                    if (!checkFileExist(configuration_1.CONFIG_NAME, options.config)) {
                        throw new error_1.SystemError("\u627E\u4E0D\u5230\u811A\u672C\u914D\u7F6E\u6587\u4EF6: " + (options.config === undefined ? configuration_1.CONFIG_NAME : options.config));
                    }
                    return [4 /*yield*/, svn.status(options.project)];
                case 1:
                    _a = _b.sent(), stdout = _a.stdout, stderr = _a.stderr;
                    if (stderr.length > 0) {
                        throw new error_1.SystemError(stderr);
                    }
                    if (!(stdout.length > 0)) return [3 /*break*/, 8];
                    changesStr = stdout.split("\r\n");
                    changes = changesStr.reduce(function (acc, cur) {
                        if (cur.length > 0) {
                            acc.push({
                                type: cur[0],
                                cwd: cur.slice(7)
                            });
                        }
                        return acc;
                    }, []);
                    return [4 /*yield*/, inquirer.prompt([
                            {
                                type: "checkbox",
                                message: "[" + configuration_1.COMMAND_NAME + "]: \u8BF7\u9009\u62E9\u63D0\u4EA4\u9879",
                                name: "items",
                                choices: changes.map(function (change) { return ({
                                    name: change.cwd,
                                    value: change.type + " " + change.cwd
                                }); }),
                                validate: function (answers) {
                                    if (answers.length === 0) {
                                        return "至少选择一项进行提交";
                                    }
                                    return true;
                                }
                            }
                        ])];
                case 2:
                    chooses = _b.sent();
                    return [4 /*yield*/, lint.check(options, chooses.items)];
                case 3:
                    lintCheckResult = _b.sent();
                    if (lintCheckResult.stderr.length > 0) {
                        throw new error_1.SystemError(lintCheckResult.stderr);
                    }
                    if (lintCheckResult.stdout.length > 0) {
                        logger.log(lintCheckResult.stdout + "\n");
                    }
                    logger.log("[" + configuration_1.COMMAND_NAME + "]: ");
                    logger.log(chalk.green("通过代码校验\n\n"));
                    return [4 /*yield*/, inquirer.prompt([
                            {
                                type: "input",
                                message: "[" + configuration_1.COMMAND_NAME + "]: \u8BF7\u586B\u5199\u63D0\u4EA4\u4FE1\u606F",
                                name: "message",
                                validate: function (message) {
                                    if (message.length === 0) {
                                        return "提交信息不为空";
                                    }
                                    return true;
                                }
                            }
                        ])];
                case 4:
                    input = _b.sent();
                    needAddItems = chooses.items.filter(function (item) {
                        return svn.NEED_ADD_STATUS.some(function (status) { return status === item.substring(0, 1); });
                    });
                    if (!needAddItems.length) return [3 /*break*/, 6];
                    return [4 /*yield*/, svn.add(needAddItems)];
                case 5:
                    _b.sent();
                    _b.label = 6;
                case 6: return [4 /*yield*/, svn.commit(chooses.items, input.message)];
                case 7:
                    _b.sent();
                    return [3 /*break*/, 9];
                case 8:
                    logger.log("[" + configuration_1.COMMAND_NAME + "]: \u6CA1\u6709\u9700\u8981\u63D0\u4EA4\u7684\u9009\u9879");
                    _b.label = 9;
                case 9: return [2 /*return*/, 0 /* OK */];
                case 10:
                    error_3 = _b.sent();
                    if (error_3.stderr) {
                        throw new error_1.SystemError(error_3.stderr);
                    }
                    if (error_3.stdout) {
                        throw new error_1.LintError(error_3.stdout);
                    }
                    throw error_3;
                case 11: return [2 /*return*/];
            }
        });
    });
}
function checkFileExist(filename, pathOrDir) {
    if (pathOrDir === undefined) {
        return fs.existsSync(filename);
    }
    return fs.existsSync(path.extname(pathOrDir).length > 0
        ? pathOrDir
        : "" + pathOrDir + path.sep + filename);
}
